////////////////////////////////////////////////////////////////////////////////
//
// A desktop-based GLSL-environment similar to the WebGL-based site
// www.shadertoy.com
//
// Copyright 2015-2016 Mirco Müller
//
// Author(s):
//   Mirco "MacSlow" Müller <macslow@gmail.com>
//
// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 3, as published
// by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranties of
// MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
// PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef _OPENGL_H
#define _OPENGL_H

#include <SDL_opengl.h>

#include "texture.h"

class OpenGL
{
	public:
		OpenGL (unsigned int width,
				unsigned int height,
				double zNear,
				double zFar);
		~OpenGL ();

		bool init (const char* imagefile0 = NULL,
				   const char* imagefile1 = NULL,
				   const char* imagefile2 = NULL,
				   const char* imagefile3 = NULL);
		bool resize (unsigned int width, unsigned int height);
		bool draw (int x, int y, int lmbx, int lmby);

	private:
		void checkGLError (const char* func);
		void dumpGLInfo ();
		GLuint loadShader (const char *src, GLenum type);
		GLuint createShaderProgram (const char* vertSrc,
									const char* fragSrc,
									bool link);
		void linkShaderProgram (GLuint progId);

	private:
		unsigned int _width;
		unsigned int _height;
        double _zNear;
        double _zFar;
        Texture* _texture[4];
		GLuint _vShaderId;
		GLuint _fShaderId;
		GLuint _program;
		GLuint _vao;
		GLuint _vbo;
		GLint _iResolution;
		GLint _iGlobaltime;
		GLint _iChannelRes[4];
		GLint _iMouse;
		GLint _iChannel[4];
		GLint _iDate;
};

#endif // _OPENGL_H
